'' =================================================================================================
''
''   File....... jm_i2c_buss_scan.spin
''   Purpose....
''   Author..... Jon "JonnyMac" McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started....
''   Updated.... 12 JUL 2016
''
'' =================================================================================================


con { timing }

  _clkmode = xtal1 + pll16x                                     
  _xinfreq = 5_000_000                                          ' use 5MHz crystal

  CLK_FREQ = (_clkmode >> 6) * _xinfreq                         ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us

  
con { io pins }

  RX1  = 31                                                     ' programming / terminal
  TX1  = 30

  SDA1 = 29                                                     ' eeprom / i2c
  SCL1 = 28

  SDA2 = 15                                                     ' for activity board test
  SCL2 = 14


con

  TEST_SDA = SDA2                                               ' buss pins to scan
  TEST_SCL = SCL2  
  

obj

' main                                                          ' * master Spin cog
  time : "jm_time"                                              '   timing and delays
  io   : "jm_io_basic"                                          '   essential io control
  i2c  : "jm_i2c"                                               '   essential i2c 
  term : "jm_fullduplexserial"                                  ' * serial io for terminal
                                                                 
' * Uses cog when loaded


var


pub main | addr, count

  setup

  ' open PST then press a key for scan

  repeat
    term.rxflush 
    term.rx
    term.tx(term#CLS)
    term.str(string("I2C Buss Scan: "))
    term.dec(TEST_SCL)
    term.tx("/")
    term.dec(TEST_SDA)
    repeat 2
      term.tx(13)
    
    count := 0
    
    repeat addr from %0000_000_0 to %1111_111_0 step 2          ' skip values with write bit set
      if (i2c.present(addr))
        format_device(addr)
        term.str(string("  $"))                                 ' show addr as 8-bit    
        term.hex(addr, 2)
        term.str(string("  0x"))                                ' show addr as 7-bit         
        term.hex(addr >> 1, 2)
        term.tx(13)
        time.pause(5)                                           ' prevent pst overflow        
        count += 1

    term.str(string(13, "Devices found: "))
    term.dec(count)

 
pub format_device(d)

  term.bin(d >> 4, 4)                                           ' device code
  term.tx("_")
  term.bin(d >> 1, 3)                                           ' address
  term.tx("_")
  term.tx("0" + (d & 1))                                        ' read/write bit


pub setup

'' Setup IO and objects for application

  time.start                                                    ' setup timing/delays for clock speed
  
  io.start($0000, $0000)                                        ' clear all pins

  i2c.setupx(TEST_SCL, TEST_SDA)                                ' connect test i2c buss

  term.start(RX1, TX1, %0000, 115_200)                          ' start serial for terminal


dat { license }

{{

  Copyright (C) 2015-2016 Jon McPhalen

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}